#!/usr/bin/env -S deno run -A

import { Command } from "https://deno.land/x/cliffy@v1.0.0-rc.4/command/mod.ts";
import { exists } from "https://deno.land/std@0.224.0/fs/exists.ts";

async function main() {
	const parsed = await new Command().name("setcores")
		.description("Sets the number of enabled cores to 'count' or just count enabled cores.")
		.option("-l, --list", "Lists available cores and states.")
		.arguments("[count: integer]")
		.parse(Deno.args);

	const cores = await collect(Core.get_possibles());

	const [count_str] = parsed.args as string[];
	if (count_str !== undefined) {
		const to_keep = parseInt(count_str);
		try {
			for (const index of range(0, to_keep)) await cores[index].set_enabled(true);
			for (const index of range(to_keep, cores.length)) await cores[index].set_enabled(false);
		} catch (_) {
			console.error("Setting core states Failed :", "\nNeeds to be run with root permission.");
		}
	}

	if (parsed.options.list) {
		for (const core of cores) {
			const name_part = core.name.padEnd(8);
			const state_part = await core.is_enabled() ? " enabled" : "disabled";
			console.log("core", name_part, "is", state_part);
		}
	}

	let enableds = 0;
	for (const core of cores) enableds += await core.is_enabled() ? 1 : 0;
	const percentage = (enableds / cores.length * 100).toString().substring(0, 5) + "%";
	console.log("total enabled", enableds, "/", cores.length, "(", percentage, ")");
}

class Core {
	name;
	directory;

	constructor(name: string, directory: string) {
		this.name = name;
		this.directory = directory;
	}

	async is_enabled() {
		try {
			const content = await cat(`${this.directory}/online`);
			return parseInt(content) === 1;
		} catch (_) {
			return true;
		}
	}

	async set_enabled(enabled: boolean) {
		const value_to_write = enabled ? "1" : "0";
		const path = `${this.directory}/online`;
		if (!await exists(path)) return false;
		await echo_into(path, value_to_write);
		return true;
	}

	static async *get_possibles() {
		const sys_cpu_dir = "/sys/devices/system/cpu";
		const possibles = await cat(`${sys_cpu_dir}/possible`);
		const [min, max] = possibles.trim().split("-").map((s) => parseInt(s));
		for (const index of range(min, max + 1)) {
			yield new Core(`${index}`, `${sys_cpu_dir}/cpu${index}`);
		}
	}
}

function* range(from: number, to: number = Infinity) {
	while (from < to) yield from++;
}

async function collect<T>(gen: AsyncIterable<T>) {
	const result = [] as T[];
	for await (const item of gen) result.push(item);
	return result;
}

async function cat(path: string) {
	const command = new Deno.Command("cat", { args: [path], stdout: "piped" });
	const result = await command.output();
	if (!result.success) throw new Error("cat Command failed");
	const decoded = new TextDecoder().decode(result.stdout);
	return decoded;
}

async function echo_into(path: string, content: string) {
	const args = ["-c", `echo '${content}' > '${path}'`];
	const command = new Deno.Command("sh", { args });
	const result = await command.output();
	if (!result.success) throw new Error("cat Command failed");
}

if (import.meta.main) await main();
